#import "HomeDetailViewController.h"

@implementation HomeDetailViewController

@synthesize toolbar;
@synthesize buttonConnect;
@synthesize	ipGCM;
@synthesize	ipiPad;

#pragma mark -
#pragma mark View lifecycle

NSString *path;

- (void)viewDidLoad {
	[super viewDidLoad];
	
	// set paths for reading last used IP address from file
	NSArray	*paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	path = [[NSString alloc] initWithString:[documentsDirectory stringByAppendingPathComponent:@"cisstIPad_ipaddress"]];
	
	// read IPs from file if exists
	if ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
		NSMutableArray *ips = [[NSMutableArray alloc] initWithContentsOfFile:path];
		[ipGCM setText:[ips objectAtIndex:0]];
		[ipiPad setText:[ips objectAtIndex:1]];
		[ips release];
	}
}

- (void)viewDidUnload {
	[super viewDidUnload];	
	self.toolbar = nil;
}

// Connect button pressed.
- (IBAction)buttonConnectPressed:(id)sender {	
	// start logger
	cmnLogger::SetMask(CMN_LOG_ALLOW_ALL);
    cmnLogger::AddChannel(std::cout, CMN_LOG_ALLOW_ALL);
	
	// write IPs to file
	NSMutableArray *ips = [[NSMutableArray alloc] initWithObjects:ipGCM.text,ipiPad.text,nil];
	[ips writeToFile:path atomically:YES];
	[ips release];
	
	// attempt to connect
	try {
		// get IP addresses from input boxes
		std::string GlobalComponentManagerAddress([ipGCM.text UTF8String]);
		std::string iPadAddress([ipiPad.text UTF8String]);
		
		// connect to global component manager
		mtsManagerLocal *local_manager = mtsManagerLocal::GetInstance(GlobalComponentManagerAddress, "iPadProcess", iPadAddress);
		std::cout << "Connected" << std::endl;
		iPadCom = new iPadInterface;
		local_manager->AddComponent(iPadCom);
		
		// create the tasks, i.e. find the commands
		local_manager->CreateAll();
		local_manager->WaitForStateAll(mtsComponentState(mtsComponentState::READY));
		
		// start the periodic Run
		local_manager->StartAll();
		local_manager->WaitForStateAll(mtsComponentState(mtsComponentState::ACTIVE));
		// success
		UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Connection Succeeded"
														message:@""
													   delegate:self
											  cancelButtonTitle:@"OK"
											  otherButtonTitles:nil];
		[alert show];
		[alert release];
		
	} catch (...) {
		// connection failed
		UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Connection Failed"
														message:@""
													   delegate:self
											  cancelButtonTitle:@"OK"
											  otherButtonTitles:nil];
		[alert show];
		[alert release];
	}
}

#pragma mark -
#pragma mark Managing the popover

- (void)showRootPopoverButtonItem:(UIBarButtonItem *)barButtonItem {
    
    // Add the popover button to the toolbar.
    NSMutableArray *itemsArray = [toolbar.items mutableCopy];
    [itemsArray insertObject:barButtonItem atIndex:0];
    [toolbar setItems:itemsArray animated:NO];
    [itemsArray release];
}


- (void)invalidateRootPopoverButtonItem:(UIBarButtonItem *)barButtonItem {
    
    // Remove the popover button from the toolbar.
    NSMutableArray *itemsArray = [toolbar.items mutableCopy];
    [itemsArray removeObject:barButtonItem];
    [toolbar setItems:itemsArray animated:NO];
    [itemsArray release];
}


#pragma mark -
#pragma mark Rotation support

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}


#pragma mark -
#pragma mark Memory management

- (void)dealloc {
    [toolbar release];
    [super dealloc];
}	


@end
