#import "AppDelegate.h"
#import "RootViewController.h"
#import "HomeDetailViewController.h"
#import "VisualDetailViewController.h"
#import "OCTDetailViewController.h"
#import "VideoDetailViewController.h"
@implementation RootViewController

@synthesize popoverController, splitViewController, rootPopoverButtonItem;

#pragma mark -
#pragma mark View lifecycle

iPadInterface *iPadCom = 0;
visStruct visConsole;
const int numComponents = 4;
int curIndex = 0;
int lastIndex = 0;
UISwipeGestureRecognizer *swipeLeft = nil;
UISwipeGestureRecognizer *swipeRight = nil;
UISwipeGestureRecognizer *swipeUp = nil;
UISwipeGestureRecognizer *swipeDown = nil;

- (void)viewDidLoad {
    
    [super viewDidLoad];
    self.contentSizeForViewInPopover = CGSizeMake(310.0, self.tableView.rowHeight*double(numComponents));
	
	// Add path of executable so that the ICE config files can be found
	NSString *envpath = [NSString stringWithFormat:@"%s;%@", getenv("PATH"), [[NSBundle mainBundle] bundlePath]];
	setenv("PATH", [envpath UTF8String], 1);
	
	// initialize component variables
	visConsole.brightness = 0;
	visConsole.contrast = 0;
	visConsole.gamma = 0;
	visConsole.magnification = 2;
	visConsole.horDispVal = 0;
	visConsole.vertDispVal = 0;
	
	// set swipe gestures
	swipeLeft = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipeLeft:)];
	swipeLeft.direction = UISwipeGestureRecognizerDirectionLeft;
	swipeLeft.numberOfTouchesRequired = 2;
	swipeRight = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipeRight:)];
	swipeRight.direction = UISwipeGestureRecognizerDirectionRight;
	swipeRight.numberOfTouchesRequired = 2;
	swipeUp = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipeUpDown:)];
	swipeUp.direction = UISwipeGestureRecognizerDirectionUp;
	swipeUp.numberOfTouchesRequired = 2;
	swipeDown = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipeUpDown:)];
	swipeDown.direction = UISwipeGestureRecognizerDirectionDown;
	swipeDown.numberOfTouchesRequired = 2;
	
	// select Home Screen in table
	NSUInteger indexes[2] = {0,curIndex};
	NSIndexPath *indexPath = [[NSIndexPath alloc] initWithIndexes:indexes length:2];
	[self.tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionNone];
	[[self.tableView delegate] tableView:self.tableView didSelectRowAtIndexPath:indexPath];

}

-(void) viewDidUnload {
	[super viewDidUnload];
	self.splitViewController = nil;
	self.rootPopoverButtonItem = nil;
}

- (void)handleSwipeLeft:(UISwipeGestureRecognizer *)recognizer {
	// select previous module
	if (curIndex > 0) {
		curIndex--;
		NSUInteger indexes[2] = {0,curIndex};
		NSIndexPath *indexPath = [[NSIndexPath alloc] initWithIndexes:indexes length:2];
		[self.tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionNone];
		[[self.tableView delegate] tableView:self.tableView didSelectRowAtIndexPath:indexPath];
	}
}

- (void)handleSwipeRight:(UISwipeGestureRecognizer *)recognizer {
	// select next module
	if (curIndex < numComponents-1) {
		curIndex++;
		NSUInteger indexes[2] = {0,curIndex};
		NSIndexPath *indexPath = [[NSIndexPath alloc] initWithIndexes:indexes length:2];
		[self.tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionNone];
		[[self.tableView delegate] tableView:self.tableView didSelectRowAtIndexPath:indexPath];
	}
}

- (void)handleSwipeUpDown:(UISwipeGestureRecognizer *)recognizer {
	// go to video feed or last used module
	curIndex = (curIndex == lastIndex) ? numComponents-1 : lastIndex;
	NSUInteger indexes[2] = {0,curIndex};
	NSIndexPath *indexPath = [[NSIndexPath alloc] initWithIndexes:indexes length:2];
	[self.tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionNone];
	[[self.tableView delegate] tableView:self.tableView didSelectRowAtIndexPath:indexPath];
}

#pragma mark -
#pragma mark Rotation support

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

//This method shows the button toolbar at the top when the application is displayed in portrait
- (void)splitViewController:(UISplitViewController*)svc willHideViewController:(UIViewController *)aViewController withBarButtonItem:(UIBarButtonItem*)barButtonItem forPopoverController:(UIPopoverController*)pc {
    
    // Keep references to the popover controller and the popover button, and tell the detail view controller to show the button.
    barButtonItem.title = @"Modules";
    self.popoverController = pc;
    self.rootPopoverButtonItem = barButtonItem;
    UIViewController <SubstitutableDetailViewController> *detailViewController = [splitViewController.viewControllers objectAtIndex:1];
    [detailViewController showRootPopoverButtonItem:rootPopoverButtonItem];
}

//This method hides the button toolbar when the application is displayed in landscape
- (void)splitViewController:(UISplitViewController*)svc willShowViewController:(UIViewController *)aViewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem {
 
    // Nil out references to the popover controller and the popover button, and tell the detail view controller to hide the button.
    UIViewController <SubstitutableDetailViewController> *detailViewController = [splitViewController.viewControllers objectAtIndex:1];
    [detailViewController invalidateRootPopoverButtonItem:rootPopoverButtonItem];
    self.popoverController = nil;
    self.rootPopoverButtonItem = nil;
}


#pragma mark -
#pragma mark Table view data source

- (NSInteger)tableView:(UITableView *)aTableView numberOfRowsInSection:(NSInteger)section {
    return numComponents;
}


- (UITableViewCell *)tableView:(UITableView *)aTableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"RootViewControllerCellIdentifier";
    
    // Dequeue or create a cell of the appropriate type.
    UITableViewCell *cell = [self.tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
    // Set appropriate labels for the cells.
	if (indexPath.row == 0)
        cell.textLabel.text = @"Home Screen";
    else if (indexPath.row == 1)
        cell.textLabel.text = @"Visualization Console";
    else if (indexPath.row == 2)
        cell.textLabel.text = @"OCT Console";
	else if (indexPath.row == 3)
		cell.textLabel.text = @"Video Feed";
	// TODO: Add additional labels for future components here

    return cell;
}


#pragma mark -
#pragma mark Table view selection

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    NSUInteger row = indexPath.row;
	curIndex = row;
	if (row != numComponents - 1)
		lastIndex = row;

	UIViewController <SubstitutableDetailViewController> *detailViewController = nil;

    if (row == 0)
		detailViewController = [[HomeDetailViewController alloc] initWithNibName:@"HomeDetailView" bundle:nil];
	else if (row == 1)
        detailViewController = [[VisualDetailViewController alloc] initWithNibName:@"VisualDetailView" bundle:nil];
    else if (row == 2)
        detailViewController = [[OCTDetailViewController alloc] initWithNibName:@"OCTDetailView" bundle:nil];
	else if (row == 3)
        detailViewController = [[VideoDetailViewController alloc] initWithNibName:@"VideoDetailView" bundle:nil];
	//TODO: Add additional cases for future components here
	//else if (row == 4 )
	//      detailViewController = [[VideoDetailViewController alloc] initWithNibName:@"VideoDetailView" bundle:nil];

	// set swipe gesture
	[detailViewController.view addGestureRecognizer:swipeLeft];
	[detailViewController.view addGestureRecognizer:swipeRight];
	[detailViewController.view addGestureRecognizer:swipeUp];
	[detailViewController.view addGestureRecognizer:swipeDown];
	
    // Update the split view controller's view controllers array.
    NSArray *viewControllers = [[NSArray alloc] initWithObjects:self.navigationController, detailViewController, nil];
    splitViewController.viewControllers = viewControllers;
    [viewControllers release];
    
    // Dismiss the popover if it's present.
    if (popoverController != nil) {
        [popoverController dismissPopoverAnimated:YES];
    }

    // Configure the new view controller's popover button (after the view has been displayed and its toolbar/navigation bar has been created).
    if (rootPopoverButtonItem != nil) {
        [detailViewController showRootPopoverButtonItem:self.rootPopoverButtonItem];
    }
	
	[detailViewController release];
}


#pragma mark -
#pragma mark Memory management

- (void)dealloc {
    [popoverController release];
    [rootPopoverButtonItem release];
    [super dealloc];
}

@end
