
#include "usrgVisRequiredInterfaces.h"
#include <cisstMultiTask/mtsComponent.h>
#include <cisstMultiTask/mtsInterfaceRequired.h>


/******************************/
/*** IReqVideoControl class ***/
/******************************/

CMN_IMPLEMENT_SERVICES_TEMPLATED(usrgDisplayType)
        CMN_IMPLEMENT_SERVICES_TEMPLATED(usrgAutoExposureSettings)
        CMN_IMPLEMENT_SERVICES_TEMPLATED(usrgBgTrackerSettings)

        IReqVideoControl::AutoExposureSettings::AutoExposureSettings() :
        Tolerance(0.05),
        Threshold(230),
        MaxGain(1305),
        MaxShutter(1000)
{
}

IReqVideoControl::AutoExposureSettings::AutoExposureSettings(double tolerance,
                                                             unsigned char threshold,
                                                             unsigned int maxgain,
                                                             unsigned int maxshutter) :
Tolerance(tolerance),
Threshold(threshold),
MaxGain(maxgain),
MaxShutter(maxshutter)
{
}

IReqVideoControl::BgTrackerSettings::BgTrackerSettings() :
        AreaRadius(600),
        FeatureSpacing(60),
        Scales(4),
        Iterations(1),
        TemplateRadius(15),
        SearchRadius(50),
        RoiSize(0.6),
        Smoothing(0.5)
{
}

IReqVideoControl::BgTrackerSettings::BgTrackerSettings(unsigned int area_radius,
                                                       unsigned int feature_spacing,
                                                       unsigned int scales,
                                                       unsigned int iterations,
                                                       unsigned int template_radius,
                                                       unsigned int search_radius,
                                                       double roi_size,
                                                       double smoothing) :
AreaRadius(area_radius),
FeatureSpacing(feature_spacing),
Scales(scales),
Iterations(iterations),
TemplateRadius(template_radius),
SearchRadius(search_radius),
RoiSize(roi_size),
Smoothing(smoothing)
{
}

IReqVideoControl::IReqVideoControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("EnableAutoExposure",           EnableAutoExposure);
            required->AddFunction("EnableBrightnessControl",      EnableBrightnessControl);
            required->AddFunction("EnableLightSourceTrigger",     EnableLightSourceTrigger);
            required->AddFunction("EnableLightSourceCompounding", EnableLightSourceCompounding);
            required->AddFunction("EnableImageMagnification",     EnableImageMagnification);
            required->AddFunction("EnableBackgroundTracking",     EnableBackgroundTracking);
            required->AddFunction("EnableTools",                  EnableTools);
            required->AddFunction("EnableAnnotations",            EnableAnnotations);
            required->AddFunction("EnableVideoStatAnnotations",   EnableVideoStatAnnotations);
            required->AddFunction("EnableBgTrackingAnnotations",  EnableBgTrackingAnnotations);
            required->AddFunction("EnableRegistrations",          EnableRegistrations);
            required->AddFunction("SetDisplayDimensions",         SetDisplayDimensions);
            required->AddFunction("SetDisplayPosition",           SetDisplayPosition);
            required->AddFunction("SetDisplayType",               SetDisplayType);
            required->AddFunction("SetDisparityCorrection",       SetDisparityCorrection);
            required->AddFunction("SetLightSourceCalibration",    SetLightSourceCalibration);
            required->AddFunction("SetLightSourceRGB",            SetLightSourceRGB);
            required->AddFunction("SetAutoExposure",              SetAutoExposure);
            required->AddFunction("SetBrightness",                SetBrightness);
            required->AddFunction("SetContrast",                  SetContrast);
            required->AddFunction("SetGamma",                     SetGamma);
            required->AddFunction("SetBackgroundTracking",        SetBackgroundTracking);
            required->AddFunction("SetMagnification",             SetMagnification);
            required->AddFunction("SetMaskThreshold",             SetMaskThreshold);
            required->AddFunction("ResetBackgroundTracking",      ResetBackgroundTracking);
            required->AddFunction("StartRecording",               StartRecording);
            required->AddFunction("PauseRecording",               PauseRecording);
            required->AddFunction("StartVideo",                   StartVideo);
            required->AddFunction("StopVideo",                    StopVideo);
            required->AddFunction("GetError",                     GetError);
        }
    }
}

IReqVideoControl::~IReqVideoControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}


/*************************************/
/*** IReqWindowEventsControl class ***/
/*************************************/

IReqWindowEventsControl::IReqWindowEventsControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("Enable",  Enable);
            required->AddFunction("Disable", Disable);
        }
    }
}

IReqWindowEventsControl::~IReqWindowEventsControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}


/***************************************/
/*** IReqJoystickEventsControl class ***/
/***************************************/

IReqJoystickEventsControl::IReqJoystickEventsControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("Enable",  Enable);
            required->AddFunction("Disable", Disable);
        }
    }
}

IReqJoystickEventsControl::~IReqJoystickEventsControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}


/*********************************/
/*** IReqOCTBasicControl class ***/
/*********************************/

IReqOCTBasicControl::IReqOCTBasicControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("SetAScanPosition", SetAScanPosition);
            required->AddFunction("SetHScanPosition", SetHScanPosition);
            required->AddFunction("ShowAScan",        ShowAScan);
            required->AddFunction("ShowHScan",        ShowHScan);
            required->AddFunction("HideAScan",        HideAScan);
            required->AddFunction("HideHScan",        HideHScan);
        }
    }
}

IReqOCTBasicControl::~IReqOCTBasicControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}




/*********************************/
/*** IReqOCTMScanControl class ***/
/*********************************/

IReqOCTMScanControl::IReqOCTMScanControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("SetMScanPosition",       SetMScanPosition);
            required->AddFunction("SetMScanSlicePosition",  SetMScanSlicePosition);
            required->AddFunction("ShowMScan",              ShowMScan);
            required->AddFunction("HideMScan",              HideMScan);
            required->AddFunction("ShowMScanSlice",         ShowMScanSlice);
            required->AddFunction("HideMScanSlice",         HideMScanSlice);
            required->AddFunction("HideAllPaths",           HideAllPaths);
            required->AddFunction("ShowCurrentPath",        ShowCurrentPath);
            required->AddFunction("ReviewModeOn",           ReviewModeOn);
            required->AddFunction("ReviewModeOff",          ReviewModeOff);
            required->AddFunction("SetUseMouseOn",          SetUseMouseOn);
            required->AddFunction("SetUseMouseOff",         SetUseMouseOff);

        }
    }
}

IReqOCTMScanControl::~IReqOCTMScanControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}


/*********************************/
/*** IReqOCTMScanGenControl class ***/
/*********************************/

IReqOCTMScanGenControl::IReqOCTMScanGenControl(const std::string& name, mtsComponent* owner) :
Name(name),
Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("Start",       Start);
			
        }
    }
}

IReqOCTMScanGenControl::~IReqOCTMScanGenControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}




/************************************/
/*** IReqRobotStatusDisplay class ***/
/************************************/

CMN_IMPLEMENT_SERVICES_TEMPLATED(usrgRobotStatus)

        IReqRobotStatusDisplay::IReqRobotStatusDisplay(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("SetStatus",   SetStatus);
            required->AddFunction("SetPosition", SetPosition);
            required->AddFunction("Show",        Show);
            required->AddFunction("Hide",        Hide);
        }
    }
}

IReqRobotStatusDisplay::~IReqRobotStatusDisplay()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}

/*************************************/
/*** IReqLaserTrackerControl class ***/
/*************************************/
/*
CMN_IMPLEMENT_SERVICES_TEMPLATED(usrgLaserTrackerParams)

        IReqLaserTrackerControl::IReqLaserTrackerControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("SetParams",    SetParams);
            required->AddFunction("Enable",       Enable);
            required->AddFunction("Disable",      Disable);
            required->AddFunction("Show",         Show);
            required->AddFunction("Hide",         Hide);
            required->AddFunction("ShowDebug",    ShowDebug);
            required->AddFunction("HideDebug",    HideDebug);
            required->AddFunction("ResetPos",     ResetPos);
            required->AddFunction("Reset",        Reset);

            required->AddFunction("SetIntensityThld",   SetIntensityThld);
            required->AddFunction("SetAdaptiveThld",    SetAdaptiveThld);
            required->AddFunction("SetPatchSize",       SetPatchSize);

            required->AddFunction("GetIntensityThld",   GetIntensityThld);
            required->AddFunction("GetAdapativeThld",   GetAdapativeThld);


        }
    }
}

IReqLaserTrackerControl::~IReqLaserTrackerControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}
*/

/******************************/
/*** IReqForceControl class ***/
/******************************/

IReqForceControl::IReqForceControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("SetPlotPosition",    SetPlotPosition);
            required->AddFunction("SetBarPosition",     SetBarPosition);
            required->AddFunction("ShowPlot",           ShowPlot);
            required->AddFunction("HidePlot",           HidePlot);
            required->AddFunction("ShowBar",            ShowBar);
            required->AddFunction("HideBar",            HideBar);
        }
    }
}

IReqForceControl::~IReqForceControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}


/*********************************************/
/*** IReqWindowEventsDebuggerControl class ***/
/*********************************************/

IReqWindowEventsDebuggerControl::IReqWindowEventsDebuggerControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("SetPosition", SetPosition);
            required->AddFunction("Show", Show);
            required->AddFunction("Hide", Hide);
        }
    }
}

IReqWindowEventsDebuggerControl::~IReqWindowEventsDebuggerControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}


/*****************************************/
/*** IReqJoystickDebuggerControl class ***/
/*****************************************/

IReqJoystickDebuggerControl::IReqJoystickDebuggerControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("Show", Show);
            required->AddFunction("Hide", Hide);
        }
    }
}

IReqJoystickDebuggerControl::~IReqJoystickDebuggerControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}


/********************************/
/*** IReqPointerControl class ***/
/********************************/

IReqPointerControl::IReqPointerControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("Show", Show);
            required->AddFunction("Hide", Hide);
        }
    }
}

IReqPointerControl::~IReqPointerControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}


/********************************/
/*** IReqPainterControl class ***/
/********************************/

IReqPainterControl::IReqPainterControl(const std::string& name, mtsComponent* owner) :
        Name(name),
        Owner(owner)
{
    if (Owner) {
        mtsInterfaceRequired* required = Owner->AddInterfaceRequired(name, MTS_OPTIONAL);
        if (required) {
            required->AddFunction("ShowCursor",   ShowCursor);
            required->AddFunction("HideCursor",   HideCursor);
            required->AddFunction("ShowPainting", ShowPainting);
            required->AddFunction("HidePainting", HidePainting);
        }
    }
}

IReqPainterControl::~IReqPainterControl()
{
    if (Owner) Owner->RemoveInterfaceRequired(Name);
}


/****************************/
/*** Stream out operators ***/
/****************************/

std::ostream & operator << (std::ostream & stream, const IReqVideoControl::DisplayType & objref)
{
    switch (objref) {
    case IReqVideoControl::Mono: stream << "Mono"; break;
    case IReqVideoControl::StereoInterlaced: stream << "StereoInterlaced"; break;
    case IReqVideoControl::StereoHorizontal: stream << "StereoHorizontal"; break;
    default: stream << "Invalid"; break;
    }
    return stream;
}

std::ostream & operator << (std::ostream & stream, const IReqVideoControl::AutoExposureSettings & objref)
{
    stream << "[" << objref.Tolerance << ", "
            << static_cast<int>(objref.Threshold) << ", "
            << objref.MaxGain << ", "
            << objref.MaxShutter << "]";
    return stream;
}

std::ostream & operator << (std::ostream & stream, const IReqVideoControl::BgTrackerSettings & objref)
{
    stream << "[" << objref.AreaRadius << ", "
            << objref.FeatureSpacing << ", "
            << objref.Scales << ", "
            << objref.Iterations << ", "
            << objref.TemplateRadius << ", "
            << objref.SearchRadius << ", "
            << objref.RoiSize << ", "
            << objref.Smoothing << "]";
    return stream;
}

std::ostream & operator << (std::ostream & stream, const IReqRobotStatusDisplay::RobotStatus & objref)
{
    stream << "[" << objref.Value << "]";
    return stream;
}
/*
std::ostream & operator << (std::ostream & stream, const IReqLaserTrackerControl::Params & objref)
{
    stream << "[" << objref.ResetPos.Left().Translation()   << ", "
            << objref.ResetPos.Right().Translation()  << ", "
            << objref.IntensityThld.Data              << ", "
            << objref.AdaptiveThld.Data               << ", "
            << objref.PatchSize[0]                    << ", "
            << objref.PatchSize[1]                    <<  "]";
    return stream;
}
*/

