
#ifndef _usrgVisRequiredInterfaces_h
#define _usrgVisRequiredInterfaces_h

#include <cisstMultiTask/mtsFunctionVoid.h>
#include <cisstMultiTask/mtsFunctionRead.h>
#include <cisstMultiTask/mtsFunctionQualifiedRead.h>
#include <cisstMultiTask/mtsFunctionWrite.h>
//#include "prmPointerPos.h"
#include <cisstMultiTask/mtsFixedSizeVectorTypes.h>


/******************************/
/*** IReqVideoControl class ***/
/******************************/

class IReqVideoControl
{
public:
    enum DisplayType
    {
        Mono             = 0,
        StereoHorizontal = 1,
        StereoInterlaced = 2
    };

    struct AutoExposureSettings
    {
        AutoExposureSettings();
        AutoExposureSettings(double tolerance,
                             unsigned char threshold,
                             unsigned int maxgain,
                             unsigned int maxshutter);

        double        Tolerance;
        unsigned char Threshold;
        unsigned int  MaxGain;
        unsigned int  MaxShutter;
    };

    struct BgTrackerSettings
    {
        BgTrackerSettings();
        BgTrackerSettings(unsigned int area_radius,
                          unsigned int feature_spacing,
                          unsigned int scales,
                          unsigned int iterations,
                          unsigned int template_radius,
                          unsigned int search_radius,
                          double       roi_size,
                          double       smoothing);

        unsigned int AreaRadius;
        unsigned int FeatureSpacing;
        unsigned int Scales;
        unsigned int Iterations;
        unsigned int TemplateRadius;
        unsigned int SearchRadius;
        double RoiSize;
        double Smoothing;
    };

    IReqVideoControl(const std::string& name, mtsComponent* owner);
    ~IReqVideoControl();

    mtsFunctionWrite EnableAutoExposure;            // bool                 ****** false
    mtsFunctionWrite EnableBrightnessControl;       // bool                 ****** false
    mtsFunctionWrite EnableLightSourceTrigger;      // bool
    mtsFunctionWrite EnableLightSourceCompounding;  // bool                 ****** false
    mtsFunctionWrite EnableImageMagnification;      // bool                 ****** false
    mtsFunctionWrite EnableBackgroundTracking;      // bool                 ****** false
    mtsFunctionWrite EnableTools;                   // bool
    mtsFunctionWrite EnableAnnotations;             // bool                 ****** false
    mtsFunctionWrite EnableVideoStatAnnotations;    // bool
    mtsFunctionWrite EnableBgTrackingAnnotations;   // bool
    mtsFunctionWrite EnableRegistrations;           // bool

    mtsFunctionWrite SetDisplayDimensions;          // vctInt2
    mtsFunctionWrite SetDisplayPosition;            // vctInt2
    mtsFunctionWrite SetDisplayType;                // DisplayType
    mtsFunctionWrite SetDisparityCorrection;        // vctInt2              ****** (0, 0) (four buttons)
    mtsFunctionWrite SetLightSourceCalibration;     // mtsDoubleMat [3x3]
    mtsFunctionWrite SetLightSourceRGB;             // vctDouble3
    mtsFunctionWrite SetAutoExposure;               // AutoExposureSettings
    mtsFunctionWrite SetBrightness;                 // double [-100, 100]   ****** 0.0 (slider)
    mtsFunctionWrite SetContrast;                   // double [-100, 100]   ****** 0.0 (slider)
    mtsFunctionWrite SetGamma;                      // double [-100, 100]   ****** 0.0 (slider)
    mtsFunctionWrite SetBackgroundTracking;         // BgTrackerSettings
    mtsFunctionWrite SetMagnification;              // double [1, 4] (steps of 0.5) ****** 2.0 (slider)
    mtsFunctionWrite SetMaskThreshold;              // unsigned char

    mtsFunctionVoid  ResetBackgroundTracking; //                            ******

    mtsFunctionVoid  StartRecording; //                                     ******
    mtsFunctionVoid  PauseRecording; //                                     ******

    mtsFunctionVoid  StartVideo; //                                         ******
    mtsFunctionVoid  StopVideo; //                                          ******
    mtsFunctionRead  GetError;
    std::string GetName(void) { return Name; }

private:
    IReqVideoControl();

    std::string Name;
    mtsComponent* Owner;
};

typedef mtsGenericObjectProxy<IReqVideoControl::DisplayType> usrgDisplayType;
CMN_DECLARE_SERVICES_INSTANTIATION(usrgDisplayType);
std::ostream & operator << (std::ostream & stream, const IReqVideoControl::DisplayType & objref);

typedef mtsGenericObjectProxy<IReqVideoControl::AutoExposureSettings> usrgAutoExposureSettings;
CMN_DECLARE_SERVICES_INSTANTIATION(usrgAutoExposureSettings);
std::ostream & operator << (std::ostream & stream, const IReqVideoControl::AutoExposureSettings & objref);

typedef mtsGenericObjectProxy<IReqVideoControl::BgTrackerSettings> usrgBgTrackerSettings;
CMN_DECLARE_SERVICES_INSTANTIATION(usrgBgTrackerSettings);
std::ostream & operator << (std::ostream & stream, const IReqVideoControl::BgTrackerSettings & objref);


/*************************************/
/*** IReqWindowEventsControl class ***/
/*************************************/

class IReqWindowEventsControl
{
public:
    IReqWindowEventsControl(const std::string& name, mtsComponent* owner);
    ~IReqWindowEventsControl();

    mtsFunctionVoid Enable;
    mtsFunctionVoid Disable;

    std::string GetName(void) { return Name; }

private:
    IReqWindowEventsControl();

    std::string Name;
    mtsComponent* Owner;
};


/***************************************/
/*** IReqJoystickEventsControl class ***/
/***************************************/

class IReqJoystickEventsControl
{
public:
    IReqJoystickEventsControl(const std::string& name, mtsComponent* owner);
    ~IReqJoystickEventsControl();

    mtsFunctionVoid Enable;
    mtsFunctionVoid Disable;

    std::string GetName(void) { return Name; }

private:
    IReqJoystickEventsControl();

    std::string Name;
    mtsComponent* Owner;
};


/*********************************/
/*** IReqOCTBasicControl class ***/
/*********************************/

class IReqOCTBasicControl
{
public:
    IReqOCTBasicControl(const std::string& name, mtsComponent* owner);
    ~IReqOCTBasicControl();

    mtsFunctionWrite SetAScanPosition;
    mtsFunctionWrite SetHScanPosition;
    mtsFunctionVoid  ShowAScan; //                                          ******
    mtsFunctionVoid  ShowHScan; //                                          ******
    mtsFunctionVoid  HideAScan; //                                          ******
    mtsFunctionVoid  HideHScan; //                                          ******

    std::string GetName(void) { return Name; }

private:
    IReqOCTBasicControl();

    std::string Name;
    mtsComponent* Owner;
};


/*********************************/
/*** IReqOCTMScanControl class ***/
/*********************************/

class IReqOCTMScanControl
{
public:
    IReqOCTMScanControl(const std::string& name, mtsComponent* owner);
    ~IReqOCTMScanControl();

    mtsFunctionWrite SetMScanPosition;
    mtsFunctionWrite SetMScanSlicePosition;
    mtsFunctionVoid  ShowMScan; //                                          ******
    mtsFunctionVoid  HideMScan; //                                          ******
    mtsFunctionVoid  ShowMScanSlice;
    mtsFunctionVoid  HideMScanSlice;

    mtsFunctionVoid  HideAllPaths;
    mtsFunctionVoid  ShowCurrentPath;

    mtsFunctionVoid  ReviewModeOn;
    mtsFunctionVoid  ReviewModeOff;

    mtsFunctionVoid  SetUseMouseOff;
    mtsFunctionVoid  SetUseMouseOn;


    std::string GetName(void) { return Name; }

private:
    IReqOCTMScanControl();

    std::string Name;
    mtsComponent* Owner;
};


/************************************/
/*** IReqOCTMScanGenControl class ***/
/************************************/

class IReqOCTMScanGenControl
{
public:
    IReqOCTMScanGenControl(const std::string& name, mtsComponent* owner);
    ~IReqOCTMScanGenControl();
	
    mtsFunctionVoid Start;	
	
    std::string GetName(void) { return Name; }
	
private:
    IReqOCTMScanGenControl();
	
    std::string Name;
    mtsComponent* Owner;
};



/************************************/
/*** IReqRobotStatusDisplay class ***/
/************************************/

class IReqRobotStatusDisplay
{
public:
    struct RobotStatus
    {
        // Customize this structure the way you want it
        double Value;
    };

    IReqRobotStatusDisplay(const std::string& name, mtsComponent* owner);
    ~IReqRobotStatusDisplay();

    mtsFunctionWrite SetStatus;
    mtsFunctionWrite SetPosition;
    mtsFunctionVoid  Show;
    mtsFunctionVoid  Hide;

    std::string GetName(void) { return Name; }

private:
    IReqRobotStatusDisplay();

    std::string Name;
    mtsComponent* Owner;
};

typedef mtsGenericObjectProxy<IReqRobotStatusDisplay::RobotStatus> usrgRobotStatus;
CMN_DECLARE_SERVICES_INSTANTIATION(usrgRobotStatus);
std::ostream & operator << (std::ostream & stream, const IReqRobotStatusDisplay::RobotStatus & objref);


/************************************/
/*** IReqForceControl class       ***/
/************************************/

class IReqForceControl
{
public:

    IReqForceControl(const std::string& name, mtsComponent* owner);
    ~IReqForceControl();

    mtsFunctionWrite SetPlotPosition;
    mtsFunctionWrite SetBarPosition;
    mtsFunctionVoid  ShowPlot;
    mtsFunctionVoid  HidePlot;
    mtsFunctionVoid  ShowBar;
    mtsFunctionVoid  HideBar;

    std::string GetName(void) { return Name; }

private:
    IReqForceControl();

    std::string Name;
    mtsComponent* Owner;
};


/*********************************************/
/*** IReqWindowEventsDebuggerControl class ***/
/*********************************************/

class IReqWindowEventsDebuggerControl
{
public:
    IReqWindowEventsDebuggerControl(const std::string& name, mtsComponent* owner);
    ~IReqWindowEventsDebuggerControl();

    mtsFunctionWrite SetPosition;
    mtsFunctionVoid  Show;
    mtsFunctionVoid  Hide;

    std::string GetName(void) { return Name; }

private:
    IReqWindowEventsDebuggerControl();

    std::string Name;
    mtsComponent* Owner;
};


/*****************************************/
/*** IReqJoystickDebuggerControl class ***/
/*****************************************/

class IReqJoystickDebuggerControl
{
public:
    IReqJoystickDebuggerControl(const std::string& name, mtsComponent* owner);
    ~IReqJoystickDebuggerControl();

    mtsFunctionVoid Show;
    mtsFunctionVoid Hide;

    std::string GetName(void) { return Name; }

private:
    IReqJoystickDebuggerControl();

    std::string Name;
    mtsComponent* Owner;
};


/********************************/
/*** IReqPointerControl class ***/
/********************************/

class IReqPointerControl
{
public:
    IReqPointerControl(const std::string& name, mtsComponent* owner);
    ~IReqPointerControl();

    mtsFunctionVoid  Show;
    mtsFunctionVoid  Hide;

    std::string GetName(void) { return Name; }

private:
    IReqPointerControl();

    std::string Name;
    mtsComponent* Owner;
};


/********************************/
/*** IReqPainterControl class ***/
/********************************/

class IReqPainterControl
{
public:
    IReqPainterControl(const std::string& name, mtsComponent* owner);
    ~IReqPainterControl();

    mtsFunctionVoid  ShowCursor;
    mtsFunctionVoid  HideCursor;
    mtsFunctionVoid  ShowPainting;
    mtsFunctionVoid  HidePainting;

    std::string GetName(void) { return Name; }

private:
    IReqPainterControl();

    std::string Name;
    mtsComponent* Owner;
};


/*************************************/
/*** IReqLaserTrackerControl class ***/
/*************************************/
/*
class IReqLaserTrackerControl
{
public:
    struct Params
    {
        // Customize this structure the way you want it
        prmPointerPos   ResetPos;
        mtsInt          IntensityThld;
        mtsInt          AdaptiveThld;
        mtsInt2         PatchSize;

    };

    IReqLaserTrackerControl(const std::string& name, mtsComponent* owner);
    ~IReqLaserTrackerControl();

    mtsFunctionWrite SetParams;

    mtsFunctionVoid  Enable;
    mtsFunctionVoid  Disable;
    mtsFunctionVoid  Show;
    mtsFunctionVoid  Hide;
    mtsFunctionVoid  ShowDebug;
    mtsFunctionVoid  HideDebug;

    mtsFunctionVoid  Reset;
    mtsFunctionWrite ResetPos;
    mtsFunctionWrite SetIntensityThld;
    mtsFunctionWrite SetAdaptiveThld;
    mtsFunctionWrite SetPatchSize;

    mtsFunctionRead  GetIntensityThld;
    mtsFunctionRead  GetAdapativeThld;

    std::string GetName(void) { return Name; }

private:
    IReqLaserTrackerControl();

    std::string Name;
    mtsComponent* Owner;
};

typedef mtsGenericObjectProxy<IReqLaserTrackerControl::Params> usrgLaserTrackerParams;
CMN_DECLARE_SERVICES_INSTANTIATION(usrgLaserTrackerParams);
std::ostream & operator << (std::ostream & stream, const IReqLaserTrackerControl::Params & objref);
*/




#endif // _usrgVisRequiredInterfaces_h

